/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.awt.Color;
import java.text.AttributedString;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPosition;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.CategoryLabelWidthType;
import org.jfree.chart.labels.PieSectionLabelGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.general.PieDataset;
import org.jfree.data.xy.TableXYDataset;
import org.jfree.data.xy.XYDataset;
import org.jfree.text.TextBlockAnchor;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.TextAnchor;

public class ChartCreator {
	public static JFreeChart createPieChart(String title, PieDataset data) {
		JFreeChart chart = ChartFactory.createPieChart(title,
				data, true, // legend?
				true, // tooltips?
				false // URLs?
				);
		
		PiePlot plot = (PiePlot)chart.getPlot();
		plot.setLabelGenerator(new PieSectionLabelGenerator() {

			@SuppressWarnings("unchecked")
			@Override
			public AttributedString generateAttributedSectionLabel(
					PieDataset dataset, Comparable key) {
				// TODO Auto-generated method stub
				return null;
			}

			@SuppressWarnings("unchecked")
			@Override
			public String generateSectionLabel(PieDataset dataset,
					Comparable key) {
				String res = ""+key;
				Number d = dataset.getValue(key);
				res = res+" "+Math.round(d.doubleValue())+"%";
				return res;
			}
			
		});
		
		return chart;
	}
	public static JFreeChart createXYChart(String title,XYDataset dataset, String subtitle) {
		JFreeChart chart = ChartFactory.createXYLineChart(
				title,	
				"time", 	//categoryAxisLabel
				subtitle,	//subtitle
				dataset,	//dataset
				PlotOrientation.VERTICAL , 	//PlotOrientation
				true,	//legend?
				true,	//Generate tool tips?
				false);	//Generate URLs?
			
		
	        chart.setBackgroundPaint(Color.white);

	        // get a reference to the plot for further customisation...
	        final XYPlot plot = chart.getXYPlot();
	        plot.setBackgroundPaint(Color.lightGray);
	        plot.setDomainGridlinePaint(Color.white);
	        plot.setRangeGridlinePaint(Color.white);
	        
	        final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
	        renderer.setSeriesLinesVisible(0, true);
	        renderer.setSeriesShapesVisible(1, true);
	        plot.setRenderer(renderer);

		return chart;
	}
	
	public static JFreeChart createStackChart(String title,
			TableXYDataset dataset) {
		JFreeChart chart = ChartFactory.createStackedXYAreaChart(
				title,	
				"time", 	//categoryAxisLabel
				"number of commits",	//title
				dataset,	//dataset
				PlotOrientation.VERTICAL , 	//PlotOrientation
				true,	//legend?
				true,	//Generate tool tips?
				false);	//Generate URLs?
			
		
	        chart.setBackgroundPaint(Color.white);

	        // get a reference to the plot for further customisation...
	        final XYPlot plot = chart.getXYPlot();
	        plot.setBackgroundPaint(Color.lightGray);
	        plot.setDomainGridlinePaint(Color.white);
	        plot.setRangeGridlinePaint(Color.white);
	        
	        final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
	        renderer.setSeriesLinesVisible(0, true);
	        renderer.setSeriesShapesVisible(1, true);
	        plot.setRenderer(renderer);

	        return chart;
	}
	public static JFreeChart create3DBarChart(String title, String category,
										String value, CategoryDataset dataset) {

		JFreeChart chart = ChartFactory.createBarChart3D(
				title,       // chart title
				category,                  // domain axis label
				value,                     // range axis label
				dataset,                     // data
				PlotOrientation.HORIZONTAL,  // orientation
				true,                        // include legend
				true,                        // tooltips
				false                        // urls
		);

		CategoryPlot plot = chart.getCategoryPlot();
		plot.setForegroundAlpha(1.0f);

		// left align the category labels...
		CategoryAxis axis = plot.getDomainAxis();
		CategoryLabelPositions p = axis.getCategoryLabelPositions();

		CategoryLabelPosition left = new CategoryLabelPosition(
				RectangleAnchor.LEFT, TextBlockAnchor.CENTER_LEFT, 
				TextAnchor.CENTER_LEFT, 0.0,
				CategoryLabelWidthType.RANGE, 0.30f
		);
		axis.setCategoryLabelPositions(CategoryLabelPositions.replaceLeftPosition(p, left));

		return chart;        

	}
}
