/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;

import org.jfree.chart.JFreeChart;
import org.jfree.data.general.DefaultPieDataset;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxAbortEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;

public class CommitAbortChart implements MultipleChartType {

	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private long totalCmt;
	private long totalAbort;
	private long totalTx;
	
	public CommitAbortChart() {
		totalCmt = 0;
		totalAbort = 0;
		totalTx = 0;
	}
	
	
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}


	@Override
	public void init(TraceFileReader traceFile) {
		reader = traceFile;
		reader.reset();
		
		Event e;
		while((e = reader.getNextEvent()) != null) {
			if (e instanceof TxCommitEvent) {
				if (((TxCommitEvent)e).tid != -1) {
					totalCmt++;
					totalTx++;
				}
			}
			else if (e instanceof TxAbortEvent) {
				if (((TxAbortEvent)e).tid != -1) {
					totalAbort++;
					totalTx++;
				}
			} 
		}
		
		DefaultPieDataset data = new DefaultPieDataset();
		data.setValue("Commits", (totalCmt/(double)totalTx)*100);
		data.setValue("Aborts", (totalAbort/(double)totalTx)*100);
		
		JFreeChart chart = ChartCreator.createPieChart("Transaction Commit/Abort Percentages", data); 
		
		charts = new ArrayList<JFreeChart>();
		charts.add(chart);
	}

}
