/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.jfree.chart.JFreeChart;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxAbortEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;


/**
 * Este Chart permite nos ver o numero de commits e aborts ao longo do tempo
 * por cada thread
 */
public class CommitAbortXYChart implements MultipleChartType{

		private final int NUMDIVISOES = 30;
		private TraceFileReader reader;
		private ArrayList<JFreeChart> charts;
		
		private class L {
			public long[] l = new long[NUMDIVISOES];
		}
		
		private HashMap<Integer,L> commits;
		private HashMap<Integer,L> aborts;
		
		public CommitAbortXYChart() {
			commits = new HashMap<Integer,L>();
			aborts = new HashMap<Integer,L>();
		}
		
		@Override
		public ArrayList<JFreeChart> getCharts() {
			return charts;
		}

		private L getCommit(int threadId) {
			L l = commits.get(threadId);
			if (l == null) {
				l = new L();
				commits.put(threadId, l);
			}
			return l;
		}
		
		private L getAbort(int threadId) {
			L l = aborts.get(threadId);
			if (l == null) {
				l = new L();
				aborts.put(threadId, l);
			}
			return l;
		}
		
		@Override
		public void init(TraceFileReader traceFile) {
			reader = traceFile;
			reader.reset();
			
			Event e = reader.getNextEvent();
			long savePoint = reader.setSavePoint();
			Long init = e.timestamp;
			e = reader.getLastEvent();
			Long end = e.timestamp;
			
			long h = (end - init)/NUMDIVISOES;
			
			reader.restoreSavePoint(savePoint);
				
			int pos = 0;
			while((e = reader.getNextEvent()) != null && pos < NUMDIVISOES-1) {
				if (e.tid != -1) {
					if(e instanceof TxCommitEvent){
						while(e.timestamp > init + (pos+1)*h)
							pos++;

						getCommit(e.threadId).l[pos]++;
					}
					else if(e instanceof TxAbortEvent){
						while(e.timestamp > init + (pos+1)*h)
							pos++;
	
						getAbort(e.threadId).l[pos]++;
					}
				}
			}
     
			final XYSeriesCollection dataset = new XYSeriesCollection();

			
			for (Map.Entry<Integer, L> com : commits.entrySet()) {
				XYSeries series = new XYSeries("Commits Thread"+com.getKey());
				for (int i=0; i < NUMDIVISOES; i++) {
					series.add(i, com.getValue().l[i]);
				}
				dataset.addSeries(series);
			}
			
			for (Map.Entry<Integer, L> com : aborts.entrySet()) {
				XYSeries series = new XYSeries("Aborts Thread"+com.getKey());
				for (int i=0; i < NUMDIVISOES; i++) {
					series.add(i, com.getValue().l[i]);
				}
				dataset.addSeries(series);
			}

			JFreeChart chart = ChartCreator.createXYChart("Commits/Aborts of Threads per time",(XYDataset)dataset,"Number of commits/aborts");
			charts = new ArrayList<JFreeChart>();
			charts.add(chart);
		}
}
