/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.StandardCategoryItemLabelGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.StackedBarRenderer;
import org.jfree.data.category.DefaultCategoryDataset;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxReadEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxWriteEvent;

public class ReadWriteTransactionChart implements MultipleChartType {

	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private HashMap<Integer,Long> reads;
	private HashMap<Integer,Long> writes;
	private HashMap<Integer,Long> total;
	
	public ReadWriteTransactionChart() {
		charts = null;
		
		reads = new HashMap<Integer,Long>();
		writes = new HashMap<Integer,Long>();
		total = new HashMap<Integer,Long>();
	}
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}

	@Override
	public void init(TraceFileReader traceFile) {
		reader = traceFile;
		reader.reset();
		
		Event e;
		while((e = reader.getNextEvent()) != null) {
			if (e instanceof TxReadEvent) {
				if (((TxReadEvent) e).tid != -1) {
					incrementTid(reads, ((TxReadEvent) e).tid);
					incrementTid(total, ((TxReadEvent) e).tid);
				}
			}
			else if (e instanceof TxWriteEvent) {
				if (((TxWriteEvent) e).tid != -1) {
					incrementTid(writes, ((TxWriteEvent) e).tid);
					incrementTid(total, ((TxWriteEvent) e).tid);
				}
			}
		}
		
		DefaultCategoryDataset data = generateDataset();
		
		JFreeChart chart = ChartFactory.createStackedBarChart(
	            "Read/Write Transaction Rates",       // chart title
	            "Individual Transactions",               // domain axis label
	            "%",                  // range axis label
	            data,                  // data
	            PlotOrientation.VERTICAL, // orientation
	            true,                     // include legend
	            true,                     // tooltips?
	            false                     // URLs?
	        );

		charts = new ArrayList<JFreeChart>();
		charts.add(chart);
		
		StackedBarRenderer renderer = (StackedBarRenderer)((CategoryPlot)chart.getPlot()).getRenderer();
		renderer.setBaseItemLabelGenerator(new StandardCategoryItemLabelGenerator());
		renderer.setBaseItemLabelsVisible(true);
	}
	
	private DefaultCategoryDataset generateDataset() {
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		
		for (Map.Entry<Integer, Long> e : reads.entrySet()) {
			dataset.addValue((e.getValue()/(double)total.get(e.getKey()))*100, 
					"Read", "Code "+e.getKey());
		}
		
		for (Map.Entry<Integer, Long> e : writes.entrySet()) {
			dataset.addValue((e.getValue()/(double)total.get(e.getKey()))*100, 
					"Write", "Code "+e.getKey());
		}
		
		return dataset;
	}
	
	private void incrementTid(HashMap<Integer,Long> map, int tid) {
		if (map.containsKey(tid)) {
			long c = map.get(tid);
			c++;
			map.put(tid, c);
		}
		else {
			map.put(tid, new Long(1));
		}
		
	}

}
