/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.jfree.chart.JFreeChart;
import org.jfree.data.general.DefaultPieDataset;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;

public class TransactionIdChart implements MultipleChartType {

	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private HashMap<Integer,Long> tids;
	
	public TransactionIdChart() {
		tids = new HashMap<Integer,Long>();
	}
	
	
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}


	@Override
	public void init(TraceFileReader traceFile) {
		reader = traceFile;
		reader.reset();
		long total = 0;
		
		Event e;
		while((e = reader.getNextEvent()) != null) {
			if (e instanceof TxCommitEvent) {
				if (((TxCommitEvent) e).tid != -1) {
					TxCommitEvent ev = (TxCommitEvent)e;
					incrementTid(ev.tid); 
					total++;
				}
			} 
		}

		DefaultPieDataset data = new DefaultPieDataset();
		
		for (Map.Entry<Integer,Long> tid : tids.entrySet()) {
			data.setValue("Code "+tid.getKey(), (tid.getValue()/(double)total)*100);
		}
		
		JFreeChart chart = ChartCreator.createPieChart("Individual Transaction Percentages", data);
		charts = new ArrayList<JFreeChart>();
		charts.add(chart);
	}



	private void incrementTid(int tid) {
		if (tids.containsKey(tid)) {
			long c = tids.get(tid);
			c++;
			tids.put(tid, c);
		}
		else {
			tids.put(tid, new Long(1));
		}
		
	}
	

}
