/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.StandardCategoryItemLabelGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxAbortEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;

/*
public class TransactionRetryChart implements MultipleChartType{

		private final int NUMDIVISOES = 30;
		private TraceFileReader reader;
		private ArrayList<JFreeChart> charts;
		
		private class L {
			public long[] l = new long[NUMDIVISOES];
		}
		
		// <transaction code, L>
		// where L[x] is the number of retries in x slice
		private HashMap<Integer,L> retries;
		
		public TransactionRetryChart() {
			retries = new HashMap<Integer,L>();
		}
		
		@Override
		public ArrayList<JFreeChart> getCharts() {
			return charts;
		}
		
		private L getRetries(int tCode) {
			L l = retries.get(tCode);
			if (l == null) {
				l = new L();
				retries.put(tCode, l);
			}
			return l;
		}
		
		@Override
		public void init(TraceFileReader traceFile) {
			reader = traceFile;
			reader.reset();
			
			Event e = reader.getNextEvent();
			long savePoint = reader.setSavePoint();
			Long init = e.timestamp;
			e = reader.getLastEvent();
			Long end = e.timestamp;
			
			long h = (end - init)/NUMDIVISOES;
			
			reader.restoreSavePoint(savePoint);
				
			int pos = 0;
			while((e = reader.getNextEvent()) != null && pos < NUMDIVISOES-1) {
				if (e.tid != -1) {
					if(e instanceof TxAbortEvent){
						while(e.timestamp > init + (pos+1)*h)
							pos++;
	
						getRetries(e.tid).l[pos]++;
					}

				}
			}
     
			final XYSeriesCollection dataset = new XYSeriesCollection();

			
			for (Map.Entry<Integer, L> com : retries.entrySet()) {
				XYSeries series = new XYSeries("Code "+com.getKey());
				for (int i=0; i < NUMDIVISOES; i++) {
					series.add(i, com.getValue().l[i]);
				}
				dataset.addSeries(series);
			}
			

			JFreeChart chart = ChartCreator.createXYChart("Transaction Retry of each code per time",(XYDataset)dataset,"Number of retries");
			charts = new ArrayList<JFreeChart>();
			charts.add(chart);
		}
}*/


public class TransactionRetryChart implements MultipleChartType {

	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private HashMap<Integer,Tx> retries;
	private final int NUMDIVISOES = 30;
	
	// <transaction code, L>
	// where L[x] is the number of retries in x slice
	private HashMap<Integer,L> retriesTime;
	
	private class Tx {
//		public boolean insideTx = false;
		public long numRetries = 0;
		public double retryAvg = 0;
		public long numTx = 0;
		public long max = 0;
	}
	private class L {
		public long[] l = new long[NUMDIVISOES];
	}
	
	public TransactionRetryChart() {
		charts = null;
		retries = new HashMap<Integer,Tx>();
		retriesTime = new HashMap<Integer,L>();
	}
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}

	@Override
	public void init(TraceFileReader traceFile) {
		reader = traceFile;
		reader.reset();
		
		Event e = reader.getNextEvent();
		long savePoint = reader.setSavePoint();
		Long init = e.timestamp;
		e = reader.getLastEvent();
		Long end = e.timestamp;
		
		long h = (end - init)/NUMDIVISOES;
		
		reader.restoreSavePoint(savePoint);
			
		int pos = 0;
		
		while((e = reader.getNextEvent()) != null ) {
			if (e.tid != -1) {
				if (e instanceof TxAbortEvent) {
					Tx tx = getTx(e.tid);
					tx.numRetries++;
					
					while(e.timestamp > init + (pos+1)*h)
					{
						pos++;
					}

					try
					{
						getRetries(e.tid).l[pos]++;
					}
					catch (Exception exc)
					{
					}
				}
				else if (e instanceof TxCommitEvent) {
					Tx tx = getTx(e.tid);
					tx.numTx++;
					tx.retryAvg += tx.numRetries;
					if (tx.numRetries > tx.max)
						tx.max = tx.numRetries;
					tx.numRetries = 0;
				}
			}
		}
		
		DefaultCategoryDataset data = generateDataset();
		
		JFreeChart chart = ChartFactory.createBarChart(
	            "Transaction Retry Rate",       // chart title
	            "Individual Transactions",               // domain axis label
	            "Number of retries",                  // range axis label
	            data,                  // data
	            PlotOrientation.VERTICAL, // orientation
	            true,                     // include legend
	            true,                     // tooltips?
	            false                     // URLs?
	        );

		charts = new ArrayList<JFreeChart>();
		charts.add(chart);
		
		BarRenderer renderer = (BarRenderer)((CategoryPlot)chart.getPlot()).getRenderer();
		renderer.setBaseItemLabelGenerator(new StandardCategoryItemLabelGenerator());
		renderer.setBaseItemLabelsVisible(true);
		////////////////////////
		final XYSeriesCollection dataset = new XYSeriesCollection();


		for (Entry<Integer, L> com : retriesTime.entrySet()) {
			XYSeries series = new XYSeries("Code "+com.getKey());
			for (int i=0; i < NUMDIVISOES; i++) {
				series.add(i, com.getValue().l[i]);
			}
			dataset.addSeries(series);
		}
		

		JFreeChart chart2 = ChartCreator.createXYChart("Transaction Retry of each code per time",(XYDataset)dataset,"Number of retries");
		charts.add(chart2);
	}
	
	private Tx getTx(int tid) {
		Tx tx = retries.get(tid);
		if (tx == null) {
			tx = new Tx();
			retries.put(tid, tx);
		}
		return tx;
	}
	
	private L getRetries(int tCode) {
		L l = retriesTime.get(tCode);
		if (l == null) {
			l = new L();
			retriesTime.put(tCode, l);
		}
		return l;
	}
	
	private DefaultCategoryDataset generateDataset() {
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		
		for (Entry<Integer, Tx> e : retries.entrySet()) {
			dataset.addValue(e.getValue().retryAvg/e.getValue().numTx, 
					"Average", "Code "+e.getKey());
			dataset.addValue(e.getValue().max, 
					"Max retries till commit", "Code "+e.getKey());
			dataset.addValue(e.getValue().retryAvg, 
					"Total", "Code "+e.getKey());
		}
		
		
		return dataset;
	}
	

}

