/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.charts;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxStartEvent;

public class TransactionsChart implements MultipleChartType {

	private TraceFileReader reader;
	private ArrayList<JFreeChart> charts;
	private HashMap<Integer,List<Tx>> threads;
	
	private class Tx {
		public int tid;
		public long start;
		public long commit;
		
		public Tx(int tid, long start) {
			this.start = start;
			this.tid = tid;
		}
	}
	
	public TransactionsChart() {
		charts = null;
		threads = new HashMap<Integer,List<Tx>>();
	}
	
	@Override
	public ArrayList<JFreeChart> getCharts() {
		return charts;
	}

	@Override
	public void init(TraceFileReader traceFile) {
		reader = traceFile;
		reader.reset();
		
		Event e;
		while((e = reader.getNextEvent()) != null) {
			if (e instanceof TxStartEvent) {
				if (((TxStartEvent) e).tid != -1) {
					addStart(e.threadId, ((TxStartEvent) e).tid, e.timestamp);
				}
			}
			else if (e instanceof TxCommitEvent) {
				if (((TxCommitEvent) e).tid != -1) {
					addCommit(e.threadId, e.timestamp);
				}
			}
		}
		
		DefaultCategoryDataset data = generateDataset();
		
		JFreeChart chart = ChartFactory.createStackedBarChart(
	            "Read/Write Transaction Rates",       // chart title
	            "Individual Transactions",               // domain axis label
	            "%",                  // range axis label
	            data,                  // data
	            PlotOrientation.HORIZONTAL, // orientation
	            true,                     // include legend
	            true,                     // tooltips?
	            false                     // URLs?
	        );
		
		charts = new ArrayList<JFreeChart>();
		charts.add(chart);
	}
	
	private DefaultCategoryDataset generateDataset() {
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		
		for (Map.Entry<Integer, List<Tx>> e : threads.entrySet()) {
			int n=1;
			for (Tx t : e.getValue()) {
				dataset.addValue(t.commit-t.start, 
					"Code "+t.tid+"x"+n, "Thread "+e.getKey());
				n++;
			}
		}
		
		
		return dataset;
	}
	
	private void addStart(int threadId, int tid, long start) {
		List<Tx> list = null;

		list = threads.get(threadId);

		if (list == null) {
			list = new ArrayList<Tx>();
			threads.put(threadId, list);
		}
		list.add(new Tx(tid, start));
	}
	
	private void addCommit(int threadId, long commit) {
		List<Tx> list = threads.get(threadId);
		Tx t = list.get(list.size()-1);
		t.commit = commit;
	}

}
