/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.plugins.sequences;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxAbortEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxMemAccessEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxReadEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxStartEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxWriteEvent;
import pt.unl.fct.di.jtraceview.util.Pair;

public class FindConflictTransaction {
	
	public static class ConflictInfo {
		TxStartEvent start;
		HashMap<Integer,List<Pair<Boolean,Long>>> concurrentThreads;
		ArrayList<TxReadEvent> transactionReadSet;				// getTransactionReadSet
		ArrayList<TxWriteEvent> transactionWriteSet;
		TxMemAccessEvent memAccessAborted;
		
		public ConflictInfo(){
			start = null;
			transactionReadSet = new ArrayList<TxReadEvent>();
			transactionWriteSet = new ArrayList<TxWriteEvent>();
			concurrentThreads = new HashMap<Integer,List<Pair<Boolean,Long>>>();
			memAccessAborted = null;
		}
	}
	
	

	public static ConflictInfo gatherInfo(Event ev, List<Integer> threads, TraceFileReader reader) {
		ConflictInfo ci = new ConflictInfo();
		
		Long s = reader.setSavePoint();
		
		Event e;
		while((e = reader.getPreviousEvent()) != null) {

			if (e.threadId == ev.threadId) {

				if (e instanceof TxStartEvent) {
					ci.start = (TxStartEvent)e;
					break;
				}
				if (e instanceof TxReadEvent) { 
					ci.transactionReadSet.add((TxReadEvent)e);
				}
				
				if (e instanceof TxWriteEvent) {
					ci.transactionWriteSet.add((TxWriteEvent)e);
				}

				if (ci.memAccessAborted == null){
					if (e instanceof TxMemAccessEvent)
						ci.memAccessAborted = (TxMemAccessEvent) e;
				}
			}
			if (e.threadId != ev.threadId) {
				if (e instanceof TxCommitEvent) {
					long sp = reader.setSavePoint();
					List<Pair<Boolean,Long>> list = ci.concurrentThreads.get(e.threadId);
					if (list == null) {
						list = new ArrayList<Pair<Boolean,Long>>();
						ci.concurrentThreads.put(e.threadId, list);
					}
					list.add(new Pair<Boolean,Long>(false, sp));
				}
				else if (e instanceof TxAbortEvent) {
					if (((TxAbortEvent)e).type == TxAbortEvent.COMMIT) {
						long sp = reader.setSavePoint();
						List<Pair<Boolean,Long>> list = ci.concurrentThreads.get(e.threadId);
						if (list == null) {
							list = new ArrayList<Pair<Boolean,Long>>();
							ci.concurrentThreads.put(e.threadId, list);
						}
						list.add(new Pair<Boolean,Long>(true, sp));
					}
				}
			}

		}

		reader.restoreSavePoint(s);
		
		HashMap<Integer,Boolean> aux = new HashMap<Integer, Boolean>();
		aux.put(ev.threadId, true);
		
		while((e = reader.getNextEvent()) != null) {
			if (e.threadId != ev.threadId) {
				if (e instanceof TxCommitEvent) {
					if (aux.get(e.threadId) == null) {
						long sp = reader.setSavePoint();
						List<Pair<Boolean,Long>> list = ci.concurrentThreads.get(e.threadId);
						if (list == null) {
							list = new ArrayList<Pair<Boolean,Long>>();
							ci.concurrentThreads.put(e.threadId, list);
						}
						list.add(new Pair<Boolean,Long>(false, sp));
					}
				}
				else if (e instanceof TxAbortEvent) {
					if (((TxAbortEvent)e).type == TxAbortEvent.COMMIT) {
						long sp = reader.setSavePoint();
						List<Pair<Boolean,Long>> list = ci.concurrentThreads.get(e.threadId);
						if (list == null) {
							list = new ArrayList<Pair<Boolean,Long>>();
							ci.concurrentThreads.put(e.threadId, list);
						}
						list.add(new Pair<Boolean,Long>(true, sp));
					}
				}
				else {
					aux.put(e.threadId, true);
				}
			}
			boolean flag = false;
			for (int th : threads) {
				if (aux.get(th) == null) {
					flag = true;
				}		
			}
			if (!flag) {
				break;
			}
			
		}
		
		reader.restoreSavePoint(s);

		return ci;
	}
	
	public static Event getConflictTransaction(TxAbortEvent ev, List<Integer> threads, TraceFileReader reader) {
		long savePoint = reader.setSavePoint();

		ConflictInfo ci = gatherInfo(ev, threads, reader);
				
		TxStartEvent start = ci.start;
		HashMap<Integer,List<Pair<Boolean,Long>>> concurrentThreads = ci.concurrentThreads;
		ArrayList<TxReadEvent> readset = ci.transactionReadSet;
		ArrayList<TxWriteEvent> writeset = ci.transactionWriteSet;
		TxMemAccessEvent mem = ci.memAccessAborted;

		if (start == null) return null;
		
		for (Map.Entry<Integer,List<Pair<Boolean,Long>>> ent : concurrentThreads.entrySet()) {
			for (Pair<Boolean,Long> p : ent.getValue()) {
				reader.restoreSavePoint(p.second);
				
				Event cm = reader.getCurrentEvent();
				TxStartEvent st = getStartTransaction(cm, reader);
				long tms = (st != null ? st.timestamp : 0);
				Event e2;
				
				while((e2 = reader.getPreviousEvent()) != null && e2.timestamp > tms) {
					if ((e2.threadId == cm.threadId) && (e2 instanceof TxWriteEvent) ){
					
						if (ev.type == TxAbortEvent.COMMIT) {
							if (intersectsWithReadset(readset, (TxWriteEvent)e2)) {
								reader.restoreSavePoint(savePoint);
								return st;
							}
							else if (intersectsWithWriteset(writeset, (TxWriteEvent)e2)) {
								reader.restoreSavePoint(savePoint);
								return st;
							}
						}
						
						if (ev.type == TxAbortEvent.OTHER) {
							if (((TxWriteEvent)e2).memCel.compareTo(mem.memCel) == 0) {
								reader.restoreSavePoint(savePoint);
								return st;
							}
						}
					}
				}
			}
		}
		
		reader.restoreSavePoint(savePoint);
		
		return null;
	}
	
	
	/**
	 * 
	 * @param ev
	 * @param threads
	 * @param reader
	 * @return 0=no conflict; 1=conflict; 2=doomed conflict; -1=error
	 */
	public static int getConflictTransaction2(TxAbortEvent ev, List<Integer> threads, TraceFileReader reader) {
		long savePoint = reader.setSavePoint();

		ConflictInfo ci = gatherInfo(ev, threads, reader);
				
		TxStartEvent start = ci.start;
		HashMap<Integer,List<Pair<Boolean,Long>>> concurrentThreads = ci.concurrentThreads;
		ArrayList<TxReadEvent> readset = ci.transactionReadSet;
		ArrayList<TxWriteEvent> writeset = ci.transactionWriteSet;
		TxMemAccessEvent mem = ci.memAccessAborted;

		if (start == null) return -1;
		
		for (Map.Entry<Integer,List<Pair<Boolean,Long>>> ent : concurrentThreads.entrySet()) {
			for (Pair<Boolean,Long> p : ent.getValue()) {
				reader.restoreSavePoint(p.second);
				
				Event cm = reader.getCurrentEvent();
				TxStartEvent st = getStartTransaction(cm, reader);
				long tms = (st != null ? st.timestamp : 0);
				Event e2;
				
				while((e2 = reader.getPreviousEvent()) != null && e2.timestamp > tms) {
					if ((e2.threadId == cm.threadId) && (e2 instanceof TxWriteEvent) ){
					
						if (ev.type == TxAbortEvent.COMMIT) {
							if (intersectsWithReadset(readset, (TxWriteEvent)e2)) {
								reader.restoreSavePoint(savePoint);
								return p.first ? 2 : 1;
							}
							else if (intersectsWithWriteset(writeset, (TxWriteEvent)e2)) {
								reader.restoreSavePoint(savePoint);
								return p.first ? 2 : 1;
							}
						}
						
						if (ev.type == TxAbortEvent.OTHER) {
							if (((TxWriteEvent)e2).memCel.compareTo(mem.memCel) == 0) {
								reader.restoreSavePoint(savePoint);
								return p.first ? 2 : 1;
							}
						}
					}
				}
			}
		}
		
		reader.restoreSavePoint(savePoint);
		
		return 0;
	}

	private static boolean intersectsWithReadset(
			ArrayList<TxReadEvent> readset, TxWriteEvent e2) {
		for (TxReadEvent r : readset) {
			if (r.memCel.compareTo(e2.memCel) == 0)
				return true;
		}
		return false;
	}
	
	private static boolean intersectsWithWriteset(
			ArrayList<TxWriteEvent> writeset, TxWriteEvent e2) {
		for (TxWriteEvent r : writeset) {
			if (r.memCel.compareTo(e2.memCel) == 0)
				return true;
		}
		return false;
	}

//	private static ArrayList<TxReadEvent> getTransactionReadSet(
//			TraceFileReader reader, TxStartEvent start) {
//		Long s = reader.setSavePoint();
//		ArrayList<TxReadEvent> readset = new ArrayList<TxReadEvent>();
//		
//		Event e;
//		while((e = reader.getPreviousEvent()) != null && e.timestamp > start.timestamp) {
//			if (e.threadId == start.threadId) {
//				if (e instanceof TxReadEvent) {
//					readset.add((TxReadEvent)e);
//				}
//			}
//		}
//		
//		reader.restoreSavePoint(s);
//		return readset;
//	}
	
	

	private static TxStartEvent getStartTransaction(Event ev,
			TraceFileReader reader) {
		Long s = reader.setSavePoint();
		TxStartEvent res = null;
		Event e;
		while((e = reader.getPreviousEvent()) != null) {
			if (e.threadId == ev.threadId) {
				if (e instanceof TxStartEvent) {
					res = (TxStartEvent)e;
					break;
				}
			}
		}
		reader.restoreSavePoint(s);
		return res;
	}

//	private static HashMap<Integer,Long> getConcurrentThreads(
//			TxStartEvent start, TraceFileReader reader) {
//		
//		Long s = reader.setSavePoint();
//		HashMap<Integer,Long> map = new HashMap<Integer,Long>();
//		
//		Event e;
//		while((e = reader.getPreviousEvent()) != null && e.timestamp > start.timestamp) {
//			if (e.threadId != start.threadId) {
//				if (e instanceof TxCommitEvent) {
//					long sp = reader.setSavePoint();
//					map.put(e.threadId, sp);
//				}
//			}
//		}
//		
//		reader.restoreSavePoint(s);
//		return map;
//	}

//	private static TxMemAccessEvent getMemAccessAborted(int threadId,
//			TraceFileReader reader) {
//		long s = reader.setSavePoint();
//		TxMemAccessEvent res = null;
//		Event e;
//		while((e = reader.getPreviousEvent()) != null) {
//			if (e.threadId == threadId) {
//				res = (TxMemAccessEvent)e;
//				break;
//			}
//		}
//		
//		reader.restoreSavePoint(s);
//		return res;
//	}
	
	public static List<Integer> getThreads(TraceFileReader reader) {
		long s = reader.setSavePoint();
		HashMap<Integer,Boolean> aux = new HashMap<Integer, Boolean>();
		List<Integer> threads = new ArrayList<Integer>();
		
		Event e;
		while((e = reader.getNextEvent()) != null) {
			aux.put(e.threadId, true);
		}
		
		for (Entry<Integer,Boolean> en : aux.entrySet()) {
			threads.add(en.getKey());
		}
		
		reader.restoreSavePoint(s);
		return threads;
	}

	
}
