/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.trace.events;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.EventId;

public class TxAbortEvent extends Event {
	
	public AbortType type;

	public TxAbortEvent(long timestamp, int threadId, int tid, AbortType type) {
		super(timestamp, EventId.TX_ABORT, threadId,tid);
		this.type = type;
	}
	
	public static class AbortType {
		private String id;
		public AbortType(String id) { this.id = id; }
		public String toString() { return id; }
		
		public static AbortType getType(byte code) {
			switch(code) {
				case 1: return USER;
				case 2: return COMMIT;
				case 3: return OTHER;
			}
			return null;
		}
	}
	
	public static AbortType getType(String type) {
		if (type.compareTo(USER.id) == 0)
			return USER;
		else if (type.compareTo(COMMIT.id) == 0)
			return COMMIT;
		else if (type.compareTo(OTHER.id) == 0)
			return OTHER;
		return null;
	}
	
	public static final AbortType USER = new AbortType("user");
	public static final AbortType COMMIT = new AbortType("commit");
	public static final AbortType OTHER = new AbortType("other");
	
	
	public String toString() {
		return super.toString()+"\t"+type;
	}

}
