/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import pt.unl.fct.di.jtraceview.trace.Event;
import pt.unl.fct.di.jtraceview.trace.TraceFileReader;
import pt.unl.fct.di.jtraceview.trace.events.TxAbortEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxCommitEvent;
import pt.unl.fct.di.jtraceview.trace.events.TxStartEvent;

/**
 * This class is used for performing modifications to the
 * trace files before being analyzed
 * 
 * @author rjfd
 *
 */
public class TracerPreProcessor {

	/**
	 * This preprocessement will be used to fix the problem of aborting 
	 * transactions on commit still have the commit event registered.
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length < 1) {
			System.out.println("Usage: TracerPreProcessor dir_path");
			System.exit(1);
		}
		
		File d = new File(args[0]);
		if (!d.isDirectory()) {
			System.out.println("The pathname must be a directory");
			System.exit(1);
		}
		
		for (File f : d.listFiles()) {
			if (!f.isDirectory()) {
				TraceFileReader reader = new TraceFileReader(f);
				PrintWriter out = null;
				File tmp = null;
				
				try {
					tmp = File.createTempFile("tfix", null);
					out = new PrintWriter(new BufferedWriter(new FileWriter(tmp)));
				} catch (IOException e1) {
					e1.printStackTrace();
					System.exit(1);
				}
				
				
				
				
				Event e;
				while((e = reader.getNextEvent()) != null) {
					if (e instanceof TxCommitEvent) {
						long s = reader.setSavePoint();
						Event e2;
						while((e2 = reader.getNextEvent()) != null) {
							if (e2.threadId == e.threadId) {
								if (e2 instanceof TxAbortEvent) {
									assert(((TxAbortEvent)e2).type == TxAbortEvent.COMMIT);
								}
								else {
									assert(e2 instanceof TxStartEvent);
									out.print(e.toString());
									out.println();
								}
								break;
							}
						}
						reader.restoreSavePoint(s);
					}
					else {
						out.print(e.toString());
						out.println();
					}
				}
				
				String fileName = f.getAbsolutePath();
				
				if (!f.delete()) {
					System.out.println("Couldn't delete file: "+f.getAbsolutePath());
					System.exit(1);
				}
				
				out.close();
				reader.close();
				
				tmp.renameTo(new File(fileName));
			}
		}

	}

}
