/*  Copyright (C) 2010  Faculdade de Ciências e Tecnologia
 *                      Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package pt.unl.fct.di.jtraceview.view.tree;

import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

@SuppressWarnings("serial")
public class ToolsTree extends JTree {
	
	private static final String ROOT_TITLE = "Tools";
	
	private DefaultMutableTreeNode root;

	public ToolsTree() {
		super();
		
		root = new DefaultMutableTreeNode(ROOT_TITLE);
		createToolsRoot();
		
		this.setModel(new DefaultTreeModel(root));
		
		expandAll();
		
		this.getSelectionModel().setSelectionMode
		(TreeSelectionModel.SINGLE_TREE_SELECTION);
		
		this.addTreeSelectionListener(new TreeSelectionListener() {

			@Override
			public void valueChanged(TreeSelectionEvent e) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)getLastSelectedPathComponent();
				if (node == null) return;
			
				if (node instanceof TreeItem) {
					TreeItem item = (TreeItem)node;
					item.itemSelected();
				}
				
			}
		});
		
	}
	
	private void createToolsRoot() {
		ChartGroupTreeItem charts = new ChartGroupTreeItem();
		root.add(charts);
		
		SequenceGroupTreeItem sequences = new SequenceGroupTreeItem();
		root.add(sequences);
	}

	private void expandAll() {
		for (int i=0; i < this.getRowCount(); i++)
			this.expandRow(i);
		
	}
}
