/*  Copyright (C) 2010 Faculdade de Ciências e Tecnologia 
 *                     Universidade Nova de Lisboa
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef EVENT_H_
#define EVENT_H_

/**
 *
 * event = 23 bytes
 * event[0-7] -> timestamp
 * event[8] -> eventId
 * event[9-10] -> threadId
 * event[11-14] -> txId
 * event[15-22] -> extra param
 */
struct event 
{
	/* Note: this structure has more then 23 bytes because of 
	 * the memory alignment.
	 */
	uint64_t timestamp;
	uint8_t id;
	uint16_t thread_id;
	uint32_t tx_id;
	union 
	{
		uint64_t addr;
		uint8_t abort_type;
	} extra;
};

#define EVENT_SIZE 23

enum tx_type
{
	TX_START=1,
	TX_COMMIT,
	TX_ABORT,
	TX_READ,
	TX_WRITE
};

extern const char const *tx_abort_type_string[];
extern const char const *tx_type_string[];

extern struct event *event_new(void);
extern struct event *event_new_set(char *);
extern void event_set(struct event *, char *);
extern char *event_get(struct event *); // the return value must be freed
extern void event_free(struct event *);

extern void event_print(struct event *);

#endif
